local Perl_Party_Pet_Events = {};

local function Perl_Party_Pet_RegisterForPet()
	this:RegisterEvent("UNIT_PVP_UPDATE");
	this:RegisterEvent("UNIT_AURA");
	this:RegisterEvent("PARTY_MEMBER_ENABLE");
	this:RegisterEvent("PARTY_MEMBER_DISABLE");
	this:RegisterEvent("UNIT_FLAGS");
	this:RegisterEvent("UNIT_DYNAMIC_FLAGS");
	Perl_RegisterBasics();
end
local function Perl_Party_Pet_UnregisterForPet()
	this:UnregisterEvent("UNIT_PVP_UPDATE");
	this:UnregisterEvent("UNIT_AURA");
	this:UnregisterEvent("PARTY_MEMBER_ENABLE");
	this:UnregisterEvent("PARTY_MEMBER_DISABLE");
	this:UnregisterEvent("UNIT_FLAGS");
	this:UnregisterEvent("UNIT_DYNAMIC_FLAGS");
	Perl_UnregisterBasics();
end


----------------------
-- Loading Function --
----------------------

function Perl_Party_Pet_OnLoad()

	--Events
	this:RegisterEvent("PLAYER_ENTERING_WORLD");
	this:RegisterEvent("PLAYER_LEAVING_WORLD");

	getglobal("Perl_Party_Pet"..this:GetID().."_DropDown"):SetID(this:GetID());
	table.insert(UnitPopupFrames,"Perl_Party_Pet"..this:GetID().."_DropDown");

	HidePartyFrame();

	this.partyunit = "party"..this:GetID();
	this.petunit = "partypet"..this:GetID();
	this.time = 0;

	Perl_InitFadeFrame(this);
end

-------------------------
-- The Update Function --
-------------------------

-- Perl_Party_Pet_CheckPet
-- returns true if full update required (frame shown)
function Perl_Party_Pet_CheckPet()
	if (Perl_Config.ShowPartyPets == 1) then
		if (UnitName(this.partyunit) and UnitName(this.petunit) and UnitIsConnected(this.petunit) and not UnitIsDeadOrGhost(this.partyunit)) then
			if (not this:IsShowing()) then
				if (UnitName("raid1")) then
					if (Perl_Config.ShowPartyRaid == 1) then
						Perl_Party_Pet_RegisterForPet();
						Perl_CancelFade(this);
						this:Show();
						return true;
					end
				else
					Perl_Party_Pet_RegisterForPet();
					Perl_CancelFade(this);
					this:Show();
					return true;
				end
			end
			return false;
		end
	end

	if (this:IsShowing()) then
		Perl_Party_Pet_UnregisterForPet();
		Perl_StartFade(this);
	end

	return false;
end

-- Perl_Party_Pet_UpdateName
local function Perl_Party_Pet_UpdateName()
	if (this.petunit == nil) then
		return;
	end

	local Partypetname = UnitName(this.petunit);

	if (Partypetname ~= nil) then
		-- Set name
		if (strlen(Partypetname) > 40) then
			Partypetname = strsub(Partypetname, 1, 39).."...";
		end
		if (Perl_Config.ShowPartyPetNames == 1) then
			getglobal(this:GetName().."_NameFrame"):Show();
		end
		getglobal(this:GetName().."_NameFrame_NameBarText"):SetText(Partypetname);

		if (UnitIsPVP(this.partyunit)) then
			getglobal(this:GetName().."_NameFrame_NameBarText"):SetTextColor(0,1,0);
		else
			getglobal(this:GetName().."_NameFrame_NameBarText"):SetTextColor(0.5,0.5,1);
		end
	end
end

-- Perl_Party_Pet_UpdateHealth
local function Perl_Party_Pet_UpdateHealth()
	local Partypethealth = UnitHealth(this.petunit);
	local Partypethealthmax = UnitHealthMax(this.petunit);

	getglobal(this:GetName().."_StatsFrame_HealthBar"):SetMinMaxValues(0, Partypethealthmax);
	getglobal(this:GetName().."_StatsFrame_HealthBar"):SetValue(Partypethealth);
	Perl_SetSmoothBarColor(getglobal(this:GetName().."_StatsFrame_HealthBar"));
	Perl_SetSmoothBarColor(getglobal(this:GetName().."_StatsFrame_HealthBarBG"),getglobal(this:GetName().."_StatsFrame_HealthBar"), 0.25);

	local phealthPct;
	phealthPct = (Partypethealth * 100.0) / Partypethealthmax
	phealthPct = string.format("%3.0f", phealthPct);

	if (UnitIsDead(this.petunit)) then
		--getglobal(this:GetName().."_StatsFrame_HealthBar_HealthBarPercentText"):SetText("Dead");
		--getglobal(this:GetName().."_StatsFrame_HealthBar_HealthBarPercentText"):Hide();
		this:Hide();
	else
		getglobal(this:GetName().."_StatsFrame_HealthBar_HealthBarPercentText"):SetText(string.format("%d",(100*(Partypethealth / Partypethealthmax))+0.5).."%");
		getglobal(this:GetName().."_StatsFrame_HealthBar_HealthBarPercentText"):Show();
	end
end

-- Perl_Party_Pet_UpdateManaType
local function Perl_Party_Pet_UpdateManaType()
	local frameName = this:GetName().."_StatsFrame_ManaBar";
	Perl_SetManaBarType(this.petunit, getglobal(frameName), getglobal(frameName.."BG"));
end

-- Perl_Party_Pet_UpdateMana
local function Perl_Party_Pet_UpdateMana()
	local Partypetmana = UnitMana(this.petunit);
	local Partypetmanamax = UnitManaMax(this.petunit);

	getglobal(this:GetName().."_StatsFrame_ManaBar"):SetMinMaxValues(0, Partypetmanamax);
	getglobal(this:GetName().."_StatsFrame_ManaBar"):SetValue(Partypetmana);

	pmanaPct = (Partypetmana * 100.0) / Partypetmanamax
	pmanaPct =  string.format("%3.0f", pmanaPct);
	getglobal(this:GetName().."_StatsFrame_ManaBar_ManaBarPercentText"):Show();
	if (UnitPowerType(this.petunit) >= 1) then
		getglobal(this:GetName().."_StatsFrame_ManaBar_ManaBarPercentText"):SetText(Partypetmana);
	else
		getglobal(this:GetName().."_StatsFrame_ManaBar_ManaBarPercentText"):SetText(string.format("%d",(100*(Partypetmana / Partypetmanamax))+0.5).."%");
	end
	if (Perl_Config.ShowPartyPetPercent == 0) then
		getglobal(this:GetName().."_StatsFrame_ManaBar_ManaBarPercentText"):Hide();
	end
end

-- Perl_Party_Pet_UpdateLevel
local function Perl_Party_Pet_UpdateLevel()
	local Partypetlevel = UnitLevel(this.petunit);
	--local PartypetClassification = UnitClassification(this.petunit);
	local color = GetDifficultyColor(Partypetlevel);

	getglobal(this:GetName().."_NameFrame_LevelBarText"):SetTextColor(color.r,color.g,color.b);

	if (Partypetlevel == 0) then
		Partypetlevel = "";
	end
	getglobal(this:GetName().."_NameFrame_LevelBarText"):SetText(Partypetlevel);
end

--------------------
-- Buff Functions --
--------------------
function Perl_Party_Pet_Buff_UpdateAll ()
	if (UnitName(this.partyunit) and UnitName(this.petunit)) then
		for buffnum=1,10 do
			local button = getglobal(this:GetName().."_BuffFrame_Buff"..buffnum);
			local icon = getglobal(button:GetName().."Icon");
			if (UnitBuff(this.petunit, buffnum)) then
				icon:SetTexture(UnitBuff(this.petunit, buffnum));
				button:Show();
			else
				button:Hide();
			end
		end
		for buffnum=1,8 do
			local button = getglobal(this:GetName().."_BuffFrame_DeBuff"..(buffnum));
			local icon = getglobal(button:GetName().."Icon");
			if (UnitDebuff(this.petunit, buffnum)) then
				local type = GameTooltipTextRight1:GetText();
				icon:SetTexture(UnitDebuff(this.petunit, buffnum));
				button:Show();
			else
				button:Hide();
			end
		end
	end
end

function Perl_Party_Pet_SetBuffTooltip ()
	local partyid = "partypet"..this:GetParent():GetParent():GetID();
	GameTooltip:SetOwner(this,"ANCHOR_BOTTOMRIGHT",30,0);
	GameTooltip:SetUnitBuff(partyid, this:GetID());
end
function Perl_Party_Pet_SetDeBuffTooltip ()
	local partyid = "partypet"..this:GetParent():GetParent():GetID();
	GameTooltip:SetOwner(this,"ANCHOR_BOTTOMRIGHT",30,0);
	GameTooltip:SetUnitDebuff(partyid, this:GetID());
end

function Perl_Party_Pet_UpdateDisplay()

	if (Perl_Party_Pet_CheckPet()) then
		Perl_Party_Pet_UpdateName();
		Perl_Party_Pet_UpdateHealth();
		Perl_Party_Pet_UpdateLevel();
		Perl_Party_Pet_UpdateManaType();
		Perl_Party_Pet_UpdateMana();
		Perl_Party_Pet_UpdateCombat();
		Perl_Party_Pet_Buff_UpdateAll();
	end
end


--------------------
-- Click Handlers --
--------------------
function Perl_Party_Pet_FindID(object)
	local name=object:GetName();
	if (strfind(name, "1")) then
		return 1;
	elseif (strfind(name, "2")) then
		return 2;
	elseif (strfind(name, "3")) then
		return 3;
	elseif (strfind(name, "4")) then
		return 4;
	else
		return nil;
	end
end
function Perl_Party_PetDropDown_OnLoad()
	UIDropDownMenu_Initialize(this, Perl_Party_PetDropDown_Initialize, "MENU");
end

function Perl_Party_PetDropDown_Initialize()
	local dropdown;
	if ( UIDROPDOWNMENU_OPEN_MENU ) then
		dropdown = getglobal(UIDROPDOWNMENU_OPEN_MENU);
	else
		dropdown = this;
	end
	UnitPopup_ShowMenu(dropdown, "PARTY", "partypet"..this:GetID());
end

function Perl_Party_Pet_MouseUp(button)

	local id=Perl_Party_Pet_FindID(this);
	this:SetID(id);
	if ( SpellIsTargeting() and button == "RightButton" ) then
		SpellStopTargeting();
		return;
	end
	if ( button == "LeftButton" ) then
		if ( SpellIsTargeting() ) then
			SpellTargetUnit("partypet"..id);
		elseif ( CursorHasItem() ) then
			DropItemOnUnit("partypet"..id);
		else
			TargetUnit("partypet"..id);
		end
	else
		if ((this:GetName())==("Perl_Party_Pet"..id)) then ToggleDropDownMenu(1, nil, getglobal(this:GetName().."_DropDown"), this:GetName(), 0, 0); end
	end

	getglobal("Perl_Party_Pet"..id):StopMovingOrSizing();
end

function Perl_Party_Pet_MouseDown(button)

end

function Perl_Party_Pet_PlayerTip()
	GameTooltip_SetDefaultAnchor(GameTooltip, this);

	if (this.petunit ~= nil) then
		GameTooltip:SetUnit(this.petunit);
	end
end

local function Perl_Target_Update_Control()
        if (UnitIsVisible(this.petunit) and UnitIsCharmed(this.petunit)) then
		getglobal(this:GetName().."_NameFrame_Warning"):Show();
	else
		getglobal(this:GetName().."_NameFrame_Warning"):Hide();
	end
end

function Perl_Party_Pet_UpdateCombat()
	if (UnitName(this.partyunit) and UnitName(this.petunit)) then
                if (UnitAffectingCombat(this.petunit)) then
                        getglobal(this:GetName().."_NameFrame_ActivityStatus"):SetTexCoord(0.5, 1.0, 0.0, 0.5);
                        getglobal(this:GetName().."_NameFrame_LevelBarText"):Hide();
                        getglobal(this:GetName().."_NameFrame_ActivityStatus"):Show();
                else
                        getglobal(this:GetName().."_NameFrame_ActivityStatus"):Hide();
                        getglobal(this:GetName().."_NameFrame_LevelBarText"):Show();
                end
		Perl_Target_Update_Control();
        end
end

function Perl_Party_Pet_OnUpdate(elapsed)
	if elapsed then
		Perl_Party_Pet_CombatFlash(elapsed, false);
		--Perl_Party_Pet_CheckPet();

		this.time = this.time + elapsed;
		if (this.time > 0.25) then
			Perl_Party_Pet_UpdateCombat();
			this.time = 0;
		end
	end

	Perl_ProcessFade(this);
end

function Perl_Party_Pet_CombatFlash(elapsed, argNew, argGreen)
	if (Perl_CombatFlashSet (elapsed, this, argNew, argGreen)) then
		Perl_CombatFlashSetFrames(getglobal(this:GetName().."_NameFrame"), this.PlayerFlash, this.PlayerFlashGreen);
		Perl_CombatFlashSetFrames(getglobal(this:GetName().."_LevelFrame"), this.PlayerFlash, this.PlayerFlashGreen);
		Perl_CombatFlashSetFrames(getglobal(this:GetName().."_StatsFrame"), this.PlayerFlash, this.PlayerFlashGreen);
	end
end


-------------------
-- Event Handler --
-------------------
local function Perl_Party_Pet_RegisterSome()
	this:RegisterEvent("PARTY_MEMBERS_CHANGED");
	this:RegisterEvent("UNIT_HAPPINESS");
	this:RegisterEvent("UNIT_PET");
end

local function Perl_Party_Pet_UnregisterSome()
	this:UnregisterEvent("PARTY_MEMBERS_CHANGED");
	this:UnregisterEvent("UNIT_HAPPINESS");
	this:UnregisterEvent("UNIT_PET");
end

function Perl_Party_Pet_OnEvent()
	local func = Perl_Party_Pet_Events[event];
	if (func) then
		func();
	else
Perl_ShowMessage("EXTRA EVENT");
	end
end

-- PLAYER_ENTERING_WORLD
function Perl_Party_Pet_Events:PLAYER_ENTERING_WORLD()
	Perl_Party_Pet_RegisterSome();
	Perl_Party_Pet_UpdateDisplay();
end

-- PLAYER_LEAVING_WORLD
function Perl_Party_Pet_Events:PLAYER_LEAVING_WORLD()
	Perl_Party_Pet_UnregisterSome();
end

-- PARTY_MEMBERS_CHANGED
function Perl_Party_Pet_Events:PARTY_MEMBERS_CHANGED()
	Perl_Party_Pet_UpdateDisplay();
end

Perl_Party_Pet_Events.UNIT_FLAGS = Perl_Party_Pet_Events.PARTY_MEMBERS_CHANGED;
Perl_Party_Pet_Events.UNIT_DYNAMIC_FLAGS = Perl_Party_Pet_Events.PARTY_MEMBERS_CHANGED;

-- UNIT_HAPPINESS
function Perl_Party_Pet_Events:UNIT_HAPPINESS()
	if (arg1 == this.petunit) then
		--Perl_Party_Pet_CheckPet();
	end
end

-- Perl_Party_Pet_Events:UNIT_COMBAT()
function Perl_Party_Pet_Events:UNIT_COMBAT()
	if (arg1 == this.petunit) then
		if (arg2 == "HEAL") then
			Perl_Party_Pet_CombatFlash(0, true, true);
		elseif (arg4 and arg4 > 0) then
			Perl_Party_Pet_CombatFlash(0, true);
		end
		--Perl_Party_Pet_CheckPet();
	end
end

-- UNIT_PET
function Perl_Party_Pet_Events:UNIT_PET()
	if (arg1 == this.partyunit) then
		Perl_Party_Pet_UpdateDisplay();
	end
end

function Perl_Party_Pet_Events:PARTY_MEMBER_ENABLE()
	if (arg1 == this.petunit or arg1 == this.partyunit) then
		Perl_Party_Pet_UpdateDisplay();
	end
end

Perl_Party_Pet_Events.PARTY_MEMBER_DISABLE = Perl_Party_Pet_Events.PARTY_MEMBER_ENABLE;

-- UNIT_NAME_UPDATE
function Perl_Party_Pet_Events:UNIT_NAME_UPDATE()
	if (arg1 == this.petunit or arg1 == this.partyunit) then
		Perl_Party_Pet_UpdateName();
	end
end

Perl_Party_Pet_Events.UNIT_PVP_UPDATE = Perl_Party_Pet_Events.UNIT_NAME_UPDATE;

-- UNIT_LEVEL
function Perl_Party_Pet_Events:UNIT_LEVEL()
	if (arg1 == this.petunit) then
		Perl_Party_Pet_UpdateLevel();
	end
end

-- UNIT_HEALTH
function Perl_Party_Pet_Events:UNIT_HEALTH()
	if (arg1 == this.petunit) then
		Perl_Party_Pet_UpdateHealth()
	end
end

-- UNIT_MAXHEALTH
function Perl_Party_Pet_Events:UNIT_MAXHEALTH()
	if (arg1 == this.petunit) then
		Perl_Party_Pet_UpdateHealth()
		Perl_Party_Pet_UpdateLevel();
	end
end

-- UNIT_AURA
function Perl_Party_Pet_Events:UNIT_AURA()
	if (arg1 == this.petunit) then
		Perl_Party_Pet_Buff_UpdateAll();
	end
end

-- UNIT_DISPLAYPOWER
function Perl_Party_Pet_Events:UNIT_DISPLAYPOWER()
	if (arg1 == this.petunit) then
		Perl_Party_Pet_UpdateManaType();
	end
end

-- UNIT_MANA
function Perl_Party_Pet_Events:UNIT_MANA()
	if (arg1 == this.petunit) then
		Perl_Party_Pet_UpdateMana();
	end
end

Perl_Party_Pet_Events.UNIT_MAXMANA	= Perl_Party_Pet_Events.UNIT_MANA;
Perl_Party_Pet_Events.UNIT_RAGE		= Perl_Party_Pet_Events.UNIT_MANA;
Perl_Party_Pet_Events.UNIT_MAXRAGE	= Perl_Party_Pet_Events.UNIT_MANA;
Perl_Party_Pet_Events.UNIT_FOCUS	= Perl_Party_Pet_Events.UNIT_MANA;
Perl_Party_Pet_Events.UNIT_MAXFOCUS	= Perl_Party_Pet_Events.UNIT_MANA;
Perl_Party_Pet_Events.UNIT_ENERGY	= Perl_Party_Pet_Events.UNIT_MANA;
Perl_Party_Pet_Events.UNIT_MAXENERGY	= Perl_Party_Pet_Events.UNIT_MANA;
